# -*- coding: utf-8 -*-
# This file is part of the party-notes module for Tryton from m-ds.de.
# The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.



# -*- coding: utf-8 -*-
# This file is part of the party-notes module for Tryton from m-ds.de.
# The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.model import fields
from trytond.pool import PoolMeta
from trytond.transaction import Transaction
from sql.conditionals import Case
from sql.functions import Function, Substring, CharLength


class Concat2(Function):
    """ concat columns
    """
    __slots__ = ()
    _function = 'concat'

# end Concat2


class Replace(Function):
    """ replace substrings
    """
    __slots__ = ()
    _function = 'replace'

# end Replace


class Party(metaclass=PoolMeta):
    __name__ = 'party.party'

    notizen = fields.Text(string='Notes')
    notizen_tree = fields.Function(fields.Char(
        string='Notes', readonly=True),
        'get_notizen_tree', searcher='search_notizen_tree')

    @classmethod
    def get_notizen_tree(cls, notes, names):
        """ get notes for selected parties
        """
        tab_party = cls.__table__()
        cursor = Transaction().connection.cursor()
        result = {x: {y.id: None for y in notes} for x in names}

        query = tab_party.select(
                    tab_party.id,
                    Case(
                        (CharLength(tab_party.notizen) > 40,
                            Concat2(Substring(Replace(
                                tab_party.notizen, '\n', '; '),
                                1, 40), '...')),
                        else_=Replace(tab_party.notizen, '\n', '; ')
                    ).as_('shorttext'),
                    where=tab_party.id.in_([x.id for x in notes]))

        cursor.execute(*query)
        records = cursor.fetchall()

        for record in records:
            values = {
                'notizen_tree': record[1],
                }
            for name in names:
                result[name][record[0]] = values[name]
        return result

    @classmethod
    def search_notizen_tree(cls, name, clause):
        """ search in fulltext
        """
        return [('notizen',) + tuple(list(clause)[1:])]

    @staticmethod
    def order_notizen_tree(tables):
        table, _ = tables[None]
        return [table.notizen]

# ende Party



# -*- coding: utf-8 -*-
# This file is part of the party-notes module for Tryton from m-ds.de.
# The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.tests.test_tryton import ModuleTestCase, with_transaction
from trytond.pool import Pool


class PartyTestCase(ModuleTestCase):
    'Test party module'
    module = 'party_notes'

    @with_transaction()
    def test_notes_create_party(self):
        """ create party, check notes, search
        """
        Party = Pool().get('party.party')

        Party.create([{
            'name': 'full name',
            'notizen': 'this is a note\nwith new line',
            }])

        party, = Party.search([])
        self.assertEqual(party.name, 'full name')
        self.assertEqual(party.notizen, 'this is a note\nwith new line')

        party2, = Party.search([('notizen', 'ilike', '%note%')])
        party2, = Party.search([('notizen_tree', 'ilike', '%note%')])

# end PartyTestCase


del ModuleTestCase



""" Tryton module to add a notes page to the party type
"""

# Always prefer setuptools over distutils
from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path
import re
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


here = path.abspath(path.dirname(__file__))
MODULE = 'party_notes'
PREFIX = 'mds'

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# tryton.cfg einlesen
config = ConfigParser()
config.readfp(open('tryton.cfg'))
info = dict(config.items('tryton'))
for key in ('depends', 'extras_depend', 'xml'):
    if key in info:
        info[key] = info[key].strip().splitlines()

# Get module-versions
modversion = {}
with open(path.join(here, 'versiondep.txt'), encoding='utf-8') as f:
    l1 = f.readlines()
    for i in l1:
        l2 = i.strip().split(';')
        if len(l2) < 4:
            continue
        modversion[l2[0]] = {'min': l2[1], 'max': l2[2], 'prefix': l2[3]}

# tryton-version
major_version = 7
minor_version = 0

requires = []
for dep in info.get('depends', []):
    if not re.match(r'(ir|res|webdav)(\W|$)', dep):
        if dep in modversion.keys():
            prefix = 'mds'
            if len(modversion[dep]['prefix']) > 0:
                prefix = modversion[dep]['prefix']

            if len(modversion[dep]['max']) > 0:
                requires.append('%s_%s >= %s, <= %s' % (
                    prefix, dep, modversion[dep]['min'],
                    modversion[dep]['max']))
            else:
                requires.append('%s_%s >= %s' % (
                    prefix, dep, modversion[dep]['min']))
        else:
            requires.append('%s_%s >= %s.%s, < %s.%s' % (
                'trytond', dep, major_version, minor_version,
                major_version, minor_version + 1))
requires.append('trytond >= %s.%s, < %s.%s' % (
    major_version, minor_version, major_version, minor_version + 1))

setup(
    name='%s_%s' % (PREFIX, MODULE),
    version=info.get('version', '0.0.1'),
    description='Tryton module to add a notes page to the party type',
    long_description=long_description,
    url='https://www.m-ds.de/',
    author='martin-data services',
    author_email='service@m-ds.de',
    license='GPL-3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Framework :: Tryton',
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Financial and Insurance Industry',
        'Topic :: Office/Business',
        'Topic :: Documentation',
        'Natural Language :: German',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],

    keywords='tryton party notes documentation',
    package_dir={'trytond.modules.%s' % MODULE: '.'},
    packages=[
        'trytond.modules.%s' % MODULE,
        ],
    package_data={
        'trytond.modules.%s' % MODULE: (
            info.get('xml', [])
            + ['tryton.cfg', 'locale/*.po', 'tests/*.py',
                'versiondep.txt', 'view/*.xml', 'README.rst']),
        },

    install_requires=requires,
    zip_safe=False,
    entry_points="""
    [trytond.modules]
    %s = trytond.modules.%s
    """ % (MODULE, MODULE),
)
